# ----

# Before running the code, make sure all packages are downloaded, and a working 
# directory is defined

# Downloading the astrochron and DecomposeR packages will be enough to download 
# the others

library(grid)
library(astrochron)
library(DecomposeR) # DecomposeR 1.0.4 at least
library(StratigrapheR) # StratigrapheR 1.2.3 at least

# Please set a working directory

setwd("C:/Users/Seb/Desktop/Publications/1. EMD/Supplementary Materials")

memento.dir <- getwd() # Directory for intermediary data saving.
                       # This is performed by the memento() function.




# 1. Signal generation ---------------------------------------------------------

# Generation of a time sequence ----

t <- seq(1, 800, 0.1) + 500 # Time sampling

# We model the deposition of two components:  a & b (see Herbert, 1994) ----

Ma <- 1 # Mean mass of component a

Sa1 <- 0.4 # Sensitivity of component a to 23 kyr precession
pa1 <- 23  # Period of precession

Sa2 <- 0.5 # Sensitivity of component a to 95 kyr eccentricity
pa2 <- 95  # Period

Sa3 <- 0.5 # Sensitivity of component a to 405 kyr eccentricity
pa3 <- 405 # Period

# Quantity of component a brought for each moment t
Fa <- Ma + 
  Sa1 * sin(2*pi*(1/pa1)*t) +
  Sa2 * sin(2*pi*(1/pa2)*t) +
  Sa3 * sin(2*pi*(1/pa3)*t)


Fa[Fa < 0] <- 0 # Avoid negative quantities

Mb <- 1 # Mean mass of component b

Sb1 <- 0.05 # Sensitivity of component b to 23 kyr precession
pb1 <- pa1  # Period of precession

Sb2 <- 0.05 # Sensitivity of component b to 95 kyr eccentricity
pb2 <- pa2  # Period

Sb3 <- 0.05 # Sensitivity of component b to 405 kyr eccentricity
pb3 <- pa3  # Period

# Quantity of component b brought for each moment t

Fb <- Mb + 
  Sb1 * sin(2*pi*(1/pb1)*t) +
  Sb2 * sin(2*pi*(1/pb2)*t) +
  Sb3 * sin(2*pi*(1/pb3)*t)

Fb[Fb < 0] <- 0 # Avoid negative quantities

# Relative accumulation of a and b ----

A <- 100 * (Fa/(Fa + Fb))

B <- 100 * (Fb/(Fa + Fb))

# We compute the stratigraphic depth (mass accumulation) ----

Tot <- Fa + Fb

Sr <- cumsum(Tot)  # Cumulative sum of total sediment, 
                   #  direct translation as depth
                   # (simplification: same density of a and b)

Sr <- round(Sr, 1) # Rounding values

# We have generated a signal "B", that evolves with the stratigraphic depth "Sr"
# This signal is now being processed





# 2. Signal analysis -----------------------------------------------------------

# Periodograms of increased time interval ----

t1 <- seq(1, 50 * 800, 0.1) + 500 # Increased time interval sampling

Fa_per1 <- Ma + 
  Sa1 * sin(2*pi*(1/pa1)*t1) +
  Sa2 * sin(2*pi*(1/pa2)*t1) +
  Sa3 * sin(2*pi*(1/pa3)*t1)

Fb_per2 <- Mb + 
  Sb1 * sin(2*pi*(1/pb1)*t1) +
  Sb2 * sin(2*pi*(1/pb2)*t1) +
  Sb3 * sin(2*pi*(1/pb3)*t1)

Fa_per1[Fa_per1 < 0] <- 0 # Avoid negative quantities

Fb_per2[Fb_per2 < 0] <- 0 # Avoid negative quantities

A_per3 <- 100 * (Fa_per1/(Fa_per1 + Fb_per2))

B_per3 <- 100 * (Fb_per2/(Fa_per1 + Fb_per2))

Tot_per3 <- Fa_per1 + Fb_per2

Sr_per3 <- cumsum(Tot_per3)  # Cumulative sum of total sediment, 
                             #  direct translation as depth
                             # (simplification: same density of a and b)

Sr_per3 <- round(Sr_per3, 1) # Rounding values

efac <- (range(Sr_per3)[2] - range(Sr_per3)[1])/(range(t1)[2] - range(t1)[1])

interp3 <- approx(x = Sr_per3, y = B_per3, xout = seq(2.7, 802386.6, 0.1))

# ----

out2 <- memento(periodogram, 
                list(data.frame(t1, Fb_per2), genplot = F,
                     output = 1, padfac = 1), 
                "Periodogram1", memento.dir)

out3 <- memento(periodogram, 
                list(data.frame(interp3$x, interp3$y), genplot = F, 
                     output = 1, padfac = 1), 
                "Periodogram2", memento.dir)

gper <- function()
{
  par(mfrow = c(2,1))


  plot(1/out2$Frequency, out2$Power/sum(out2$Power),
       type = "l", log = "x", xlim = c(5, 800),
       xlab = "Period (kyr)", ylab = "Relative power",
       las = 1)

  abline(v = c(23, 95, 405), col = "red")

  plot(1/out3$Frequency, out3$Power/sum(out3$Power),
       type = "l", log = "x", xlim = efac * c(5, 800),
       xlab = "Period (arbitrary depth units)", ylab = "Relative power",
       las = 1)

  abline(v = efac * c(23, 95, 405), col = "red")

  par(mfrow = c(1,1))
}

pdfDisplay(gper(), "ADC Periodograms", wi = 6, he = 8, track = F, dir = getwd())






# Perform Singular Spectrum Analysis (SSA) ----

ss <- memento(simple.ssa, list(B, Sr, n = 20), "SSA1", memento.dir)

plot_emd(ss, style = 1, name = "SSA", track = F, dir = getwd(), 
         s = list(type = "l", lwd = 3), 
         i = list(type = "l", lwd = 3))

check.emd(ss, xy = B)





# Perform Ensemble Empirical Mode Decomposition (EEMD)----

set.seed(42)

se <- memento(extricate, list(B, Sr, nimf = 9, 
                sifting = 2,
                factor_noise = 100, 
                unit_noise = "native",
                comb = 200,
                speak = T), "EMD1", memento.dir)

plot_emd(se, style = 1, track = F,  dir = getwd(), 
         s = list(type = "l", lwd = 3), 
         i = list(type = "l", lwd = 3))

check.emd(se, xy = B)





# Regrouping the components of the EEMD for better meaningfulness ----

de1 <- mode.bind(se, mode = c(1:4),adjust = F, name = "bound")
de2 <- mode.bind(de1, mode = c(2:3),adjust = F, name = "bound")
de3 <- mode.bind(de2, mode = c(4:5),adjust = F, name = "bound")
de4 <- mode.bind(de3, mode = c(5:6),adjust = F, name = "bound")

plot_emd(de4, style = 1, name = "Grouped EMD", track = F)

check.emd(de4, xy = B)



# Instantaneous parameters analysis on the "grouped" EEMD ----

# inst.pulse computes the instantaneous frequency and amplitude

pu1 <- memento(inst.pulse, list(de4, plot = F), "RATIO1", memento.dir)

gpu1 <- function() plot_pulse(pu1, lines = efac * c(23, 95, 405))

pdfDisplay(gpu1(), "Spectra ADC", dir = getwd(), 
           width = 8, height = 5, track = F)

# inst.raio computes the instantaneous freqeuncy ratios

r1 <- inst.ratio(pu1, plot = F)

r1a <- plot_ratio(r1, style = "s", lines = c(95/23, 405/23), 
                  plot = F)
r1b <- plot_ratio(r1, style = "s", lines = c(95/23), select = "2/3", plot = F)
r1c <- plot_ratio(r1, style = "s", lines = c(405/23), select = "2/4", plot = F)
r1d <- plot_ratio(r1, style = "s", lines = c(405/95), select = "3/4", plot = F)

# Arrange the ratios plots ----

gr1 <- function()
{
  
  grid.newpage()
  
  lay <- grid.layout(ncol = 1, nrow = 4)
  
  vp1 <- viewport(layout = lay, name = "vp1")
  
  pushViewport(vp1)
  
  pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
  
  grid.draw(r1a)
  
  seekViewport("vp1")
  
  pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))
  
  grid.draw(r1b)
  
  seekViewport("vp1")
  
  pushViewport(viewport(layout.pos.row = 3, layout.pos.col = 1))
  
  grid.draw(r1c)
  
  seekViewport("vp1")
  
  pushViewport(viewport(layout.pos.row = 4, layout.pos.col = 1))
  
  grid.draw(r1d)
  
  seekViewport("vp1")
  
  
}

pdfDisplay(gr1(), name = "ADC ratios", 
           width = 8, height = 20, track = F, dir = getwd())




