# ----

# Before running the code, make sure all packages are downloaded, and a working 
# directory is defined

# Downloading the astrochron and DecomposeR packages will be enough to download 
# the others

library(grid)
library(DecomposeR) # DecomposeR 1.0.4 at least
library(StratigrapheR) # StratigrapheR 1.2.3 at least

# Please set a working directory

setwd("C:/Users/Seb/Desktop/Publications/1. EMD/Supplementary Materials")

memento.dir <- getwd() # Directory for intermediary data saving.
                       # This is performed by the memento() function.


# Decomposition ----

xy <- cip1$xy
dt <- cip1$dt

set.seed(42)

dec <- memento(extricate, 
               list(xy, dt, nimf = 7, repl = 10, comb = 300, sifting = 1, 
                    bind = T, factor_noise = 100, unit_noise = "native", 
                    speak = T),
               "CIP1_EMD1", memento.dir)

check.emd(dec, xy = xy)

integrity(xy, dec)

cd1 <- parsimony(dec)

mean(cd1)

# Plot decomposition ----

plot_emd(repl.out(dec, keep = 1),  track = F, 
         # adapt.axis = T,
         xylim = c(200, 0), 
         dtlim = c(max(dt) + 0.1, min(dt) - 0.1), style = 1, size.xy = 3.5, 
         s = list(type = "l"), i = list(type = "l"),
         li = list(v = 0, col = "black"), box = F, parg = list(yaxs = "i"), 
         ext = ".pdf", openfile = T, dir = getwd())



# Plot instantaneous frequency of decomposition ----

m <- c(23, 125, 405, 1000, 2400) / 75.81

ht <- inst.pulse(dec, plot = F)

id1 <- gzc.departure(pulse = ht)

colMeans(id1)

gp1 <- function() plot_pulse(ht, lines = m)
  
pdfDisplay(gp1(), "emdf", width = 8, height = 5, track = F, dir = getwd())

# Plot instantaneous ratios of frequencies of decomposition ----

r1 <- inst.ratio(ht, plot = F)

r1a <- plot_ratio(r1, style = "s", lines = c(125/23, 405/23, 1000/23, 2400/23), 
                  plot = F)
r1b <- plot_ratio(r1, style = "s", lines = c(125/23), select = "2/4", plot = F)
r1c <- plot_ratio(r1, style = "s", lines = c(405/23), select = "2/5", plot = F)
r1d <- plot_ratio(r1, style = "s", lines = c(1000/23), select = "2/6", plot = F)

gr1 <- function()
{
  
  grid.newpage()
  
  lay <- grid.layout(ncol = 1, nrow = 4)
  
  vp1 <- viewport(layout = lay, name = "vp1")
  
  pushViewport(vp1)
  
  pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
  
  grid.draw(r1a)
  
  seekViewport("vp1")
  
  pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))
  
  grid.draw(r1b)
  
  seekViewport("vp1")
  
  pushViewport(viewport(layout.pos.row = 3, layout.pos.col = 1))
  
  grid.draw(r1c)
  
  seekViewport("vp1")
  
  pushViewport(viewport(layout.pos.row = 4, layout.pos.col = 1))
  
  grid.draw(r1d)
  
  seekViewport("vp1")
  
  
}

pdfDisplay(gr1(), name = "Precession ratios", width = 8, height = 20, 
           track = F, dir = getwd())

# Extract supposed precession ----

pre <- repl.out(dec, keep = 1)$m[,2]

gpre <- gzc(repl.out(dec, keep = 1))

mod <- normalise(m = pre, dt = dt, last = T, speak = T)$a

# Plot different instantaneous frequency and amplitude determination methods ----

normf <- inst.pulse(m = pre, dt = dt, last = T, method = "NHT", plot = F)

htf <- inst.pulse(m = pre, dt = dt, last = T, plot = F)

dep <- gzc.departure(htf, simplify = F)[,1]

gf <- function()
{
  
  par(mfrow = c(1,3))
  
  ylim <- c(max(cip1$dt) + 0.1, 15.1)
  
  plot(pre, dt, type = "l", pch = 19, col = "grey", axes = F, 
       xlab = "Arbitrary Units", ylab = "Depth (m)",
       xlim = c(100, -100), ylim = ylim, 
       yaxs = "i")
  # lines(-ht$a[,2], dt, col = "blue")
  # lines(-mod, dt, col = "red", lwd = 1)
  # lines(-gpre$a[,2], gpre$dt[,2], col = "darkgreen", lwd = 2, lty = 1)
  
  abline(v = 0)
  
  axis(1)
  axis(2)
  
  plot(1/ht$f[,2], dt, col = "blue", type = "l", log = "x", xlim = c(0.01, 100), 
       ylim = ylim, axes = F, 
       xlab = "Period", ylab = "",
       yaxs = "i")
  lines(1/normf$f, dt, col = "red", lwd = 1)
  lines(1/gpre$f[,2], gpre$dt[,2], col = "darkgreen", lwd = 1.5, lty = 1)
  
  abline(v = 23/75.81)
  
  as <- seq_log(0.01, 100, divide = T)
  
  minorAxis(1, at.maj = as[[1]], 
            labels.maj = formatC(as[[1]], format = "g", digits = 3), 
            at.min = as[[2]])
  
  plot(dep, dt, ylim = ylim, axes = F, yaxs = "i", type = "l", 
       xlab = "IMF Departure", ylab = "", xlim = c(-10,10))
  
  axis(1)
  
  par(mfrow = c(1,1))

}

pdfDisplay(gf(), "F comparison", height = 7, width = 7, 
           track = F, dir = getwd())




# Decompose supposed precession demodulation ----

set.seed(42)

dem <- extricate(mod, dt, nimf = 6, ini = pre, sifting = 1,
                 repl = 10, comb = 100, factor_noise = 200,
                 unit_noise = "native", speak = T)

check.emd(dem, xy = mod)

integrity(mod, dem)

cd2 <- parsimony(dem)

mean(cd2)

# Plot decomposition of supposed precession demodulation ----

plot_emd(repl.out(dem,keep = 1), track = F,
         inilim = c(max(mod), -max(mod)),
         dtlim = c(max(dt) + 0.1, min(dt) - 0.1), size.xy = 3.5, 
         s = list(type = "l"), 
         i = list(type = "l"), e = list(col = "black", lwd = 1),
         li = list(v = 0, col = "black"), box = F, parg = list(yaxs = "i"), 
         style = 1, name = "DEM", dir = getwd())

# Plot instantaneous frequency of decomposition of supposed precession demodulation ----

m2 <- c(23, 125, 405, 1000, 2400) / 75.81

htr <- inst.pulse(dem, plot = F)

id2 <- gzc.departure(htr)

colMeans(id2)

gp2 <- function() plot_pulse(htr, lines = m2)

pdfDisplay(gp2(), "demf", dir = getwd(), width = 8, height = 5, track = F)

# Plot instantaneous ratios of frequencies of decomposition of supposed precession demodulation ----

r2 <- inst.ratio(htr, plot = F)

r2a <- plot_ratio(r2, style = "s", lines = c(125/23, 405/23, 1000/23, 2400/23), 
                  plot = F)
r2b <- plot_ratio(r2, style = "s", lines = c(125/23), select = "1/5", plot = F)
r2c <- plot_ratio(r2, style = "s", lines = c(405/23), select = "1/6", plot = F)
r2d <- plot_ratio(r2, style = "s", lines = c(1000/23), select = "1/7", plot = F)

gr2 <- function()
{

  grid.newpage()

  lay <- grid.layout(ncol = 1, nrow = 4)

  vp1 <- viewport(layout = lay, name = "vp1")

  pushViewport(vp1)

  pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))

  grid.draw(r2a)

  seekViewport("vp1")

  pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))

  grid.draw(r2b)

  seekViewport("vp1")

  pushViewport(viewport(layout.pos.row = 3, layout.pos.col = 1))

  grid.draw(r2c)

  seekViewport("vp1")

  pushViewport(viewport(layout.pos.row = 4, layout.pos.col = 1))

  grid.draw(r2d)

  seekViewport("vp1")


}

pdfDisplay(gr2(), name = "Precession amplitude ratios", 
           width = 8, height = 20, track = F, dir = getwd())


# Compare phase between components and their equivalent in demodulation ----

emd1 <- repl.out(dec, keep = 1)$m
dem1 <- repl.out(dem, keep = 1)$m

gph <- function()
{
  
  par(mfrow = c(1,6), mar = c(5,4,4,2))
  
  plot(-emd1[,4], dt, type = "l", ylim = c(max(dt), min(dt)), xlim = c(-25, 25),
       axes = F, xlab = "Arbitrary Units", ylab = "Depth (m)")
  lines(dem1[,4], dt, type = "l", col = "red")
  abline(v = 0)
  
  axis(1)
  axis(2)
  
  plot(-emd1[,5], dt, type = "l", ylim = c(max(dt), min(dt)), xlim = c(-25, 25),
       axes = F, xlab = "", ylab = "")
  lines(dem1[,5], dt, type = "l", col = "red")
  abline(v = 0)
  
  plot(-emd1[,6], dt, type = "l", ylim = c(max(dt), min(dt)), xlim = c(-25, 25), 
       axes = F, xlab = "", ylab = "")
  lines(dem1[,6], dt, type = "l", col = "red")
  abline(v = 0)
  
  plot(-emd1[,6] * 3, dt, type = "l", ylim = c(max(dt), min(dt)), xlim = c(-25, 25), 
       axes = F, xlab = "", ylab = "")
  lines(dem1[,6] * 3, dt, type = "l", col = "red")
  abline(v = 0)
  
  plot(-emd1[,7], dt, type = "l", ylim = c(max(dt), min(dt)), xlim = c(-25, 25), 
       axes = F, xlab = "", ylab = "")
  lines(dem1[,7], dt, type = "l", col = "red")
  abline(v = 0)
  
  
  plot(-emd1[,7] * 6, dt, type = "l", ylim = c(max(dt), min(dt)), xlim = c(-25, 25), 
       axes = F, xlab = "", ylab = "")
  lines(dem1[,7] * 6, dt, type = "l", col = "red")
  abline(v = 0)
  
}

pdfDisplay(gph(), "Comparison phase",track = F, width = 10, height = 10, 
           dir = getwd())


# Periodogram of the insolation solution used as input to the CIP1 case ----

a2 <- cip1_input

dt <- a2$age
xy <- a2$xy - mean(a2$xy)

out <- periodogram(data.frame(dt = dt, xy = xy), padfac = 1, 
                   output = 1, genplot = F)

keep <- 1/out$Frequency <= 2002  & 1/out$Frequency >= 10

length(out$Frequency)

# Periodogram of the demodulation of precession from 0 to 20 Ma ----

d2 <- La04_pre_0_20

dt2 <- d2$age
xy2 <- d2$xy - mean(d2$xy)

norm <- normalise(m = xy2, dt = dt2)

amp <- norm$a - mean(norm$a)

out2 <- periodogram(data.frame(dt = dt2, xy = amp), padfac = 10,
                    output = 1, genplot = F)

keep2 <- 1/out2$Frequency <= 5002 & 1/out2$Frequency >= 50


# Plot both periodograms ----

g <- function()
{
  
  par(mfrow = c(2,1))
  
  plot(1/out$Frequency[keep], out$Power[keep]/sum(out$Power[keep]),
       type = "l", log = "x", xlim = c(10, 5000), lwd = 1,
       main = "Power spectra of the insolation",
       xlab = "Period", ylab = "Relative power", axes = F)
  
  axl <- seq_log(2, 5000, divide = T)
  
  axis(2, las = 1)
  minorAxis(1, at.maj = axl[[1]], at.min = axl[[2]])
  
  abline(v = c(19, 23, 40), col = "red")
  
  # ----
  
  plot(1/out2$Frequency[keep2], out2$Power[keep2]/sum(out2$Power[keep2]),
       type = "l", log = "x", xlim = c(10, 5000), lwd = 1,
       main = "Power spectra of the demodulation of climatic precession (0 to 20 Ma)",
       xlab = "Period", ylab = "Relative power", axes = F)
  
  axis(2, las = 1)
  minorAxis(1, at.maj = axl[[1]], at.min = axl[[2]])
  
  abline(v = c(95, 125, 405, 1000, 2400), col = "red")
}

# g()

pdfDisplay(g(), name = "PS Solutions", dir = getwd(), height = 7, width = 7, track = F)




# Generate reference curves, and compare them to the signal----

o2 <- La04_obl_6_8

ylim <- c(8000, 6000)

keep3 <- d2$age <= 8000 & d2$age >= 6000

# ----

# Decomposition and filtering of CIP1, same parameters as in other codes ----

set.seed(42)

dec <- extricate(cip1$xy, cip1$dt, nimf = 5, repl = 1, comb = 300, sifting = 1,
                 factor_noise = 100, unit_noise = "native", speak = T)

# set.seed(42)
# 
# dec <- memento(extricate, 
#                list(xy, dt, nimf = 7, repl = 10, comb = 300, sifting = 1, 
#                     bind = T, factor_noise = 100, unit_noise = "native", 
#                     speak = T),
#                "CIP1_EMD1", memento.dir)


filpre <- taner(data.frame(dt = cip1$dt, xy = cip1$xy - mean(cip1$xy)),
                flow = 75.81/15, fhigh = 75.81/30, genplot = F)

# Filtering of solutions ----

filo <- taner(data.frame(dt = a2$age, xy = a2$xy - mean(a2$xy)),
              flow = 1/100, fhigh = 1/35, genplot = F)

filp <- taner(data.frame(dt = a2$age, xy = a2$xy - mean(a2$xy)),
              flow = 1/35, fhigh = 1/15, genplot = F)

remain <- a2$xy - filo$xy - filp$xy - mean(a2$xy)

m <- matrix(c(filp$xy, filo$xy, remain, rep(mean(a2$xy), length(remain))),
            ncol = 4)

ins <- as.emd(dt = a2$age, imf = m, xy = a2$xy)

# plot_emd(ins, track = F)

integrity(xy = a2$xy, ins)

parsimony(ins)

bulk <- sum(abs(a2$xy -  mean(a2$xy)))

pp <- 100 * sum(abs(filp$xy))/bulk
po <- 100 * sum(abs(filo$xy))/bulk
pr <- 100 * sum(abs(remain))/bulk

added <- filp$xy + filo$xy

pa <- 100 * sum(abs(added))/bulk

pp
po
pr

sum(c(pp, po, pr))



# ----

no <- o2$xy - mean(o2$xy)

coo <- lm(filo$xy ~ no)

np <- d2$xy[keep3] - mean(d2$xy[keep3])

cop <- lm(filp$xy ~ np)



# LOCAL PARSIMONY TEST ----

# Decomposition comp.2 ----

tmean <- mean(cip1$xy)

mat1 <- matrix(c(dec$m[,2], cip1$xy - dec$m[,2] - tmean), ncol = 2)

t1000 <- as.emd(xy = cip1$xy - tmean, dt = cip1$dt, imf = mat1)

# plot_emd(t1000, track = F, name = "t1000")

integrity(cip1$xy - tmean, t1000)

parsimony(t1000, correct = NULL)

# locpa1 <- parsimony(t1000, correct = NULL, bulk = F)
#
# locpa3 <- abs(t1000$m[,1]) + abs(t1000$m[,2]) - abs(t1000$xy)

# Filter ----

mat2 <- matrix(c(filpre$xy, cip1$xy - filpre$xy - tmean), ncol = 2)

t2000 <- as.emd(xy = cip1$xy - tmean, dt = cip1$dt, imf = mat2)

# plot_emd(t2000, track = F, name = "t2000")

integrity(cip1$xy - tmean, t2000)

parsimony(t2000, correct = NULL)



sum(abs(dec$m[,2]))
sum(abs(filpre$xy))

100 * (sum(abs(filpre$xy)) - sum(abs(dec$m[,2])))/sum(abs(dec$m[,2]))






# Plotting the curves to compare precession to the signal ----

g <- function()
{
  par(mfrow = c(1,7))
  
  plot(filp$xy, filp$dt, axes = F, ylim = ylim, xlim = c(-50, 50), type = "l",
       ylab = "Age (kyr)", xlab = "W/m squared")
  
  fp <- - cop$coefficients[2]
  
  lines(-fp * (d2$xy[keep3] - mean(d2$xy[keep3])), d2$age[keep3],
        type = "l", col = "blue", lty = 1)
  
  mp <- c(-0.04, -0.02, 0, 0.02, 0.04)
  
  axis(3, at = fp * (mp -  mean(d2$xy[keep3])), labels = mp)
  
  axis(2)
  axis(1)
  
  plot(filo$xy, filo$dt, axes = F, ylim = ylim, type = "l",
       ylab = "", xlab = "W/m squared")
  
  of <- coo$coefficients[2]
  
  lines(of * (o2$xy - mean(o2$xy)), o2$age,
        type = "l", col = "blue", lty = 1)
  
  mo <- c(0.39, 0.40, 0.41, 0.42)
  
  axis(3, at = of * (mo - mean(o2$xy)), labels = mo)
  
  axis(1)
  
  plot(filo$xy, filo$dt, axes = F, ylim = ylim, xlim = c(-50, 50), type = "l",
       ylab = "", xlab = "W/m squared")
  
  axis(1)
  
  plot(a2$xy, a2$age, ylim = ylim, type = "l", axes = F, main = "Mean Insolation",
       xlab = "W/m squared", ylab = "", xlim = c(420, 520))
  
  axis(1)
  
  # ----
  
  plot(cip1$xy, cip1$dt, type = "l", axes = F, main = "Signal",
       xlim = c(200, 0), ylim = c(max(cip1$dt), 0),
       xlab = "Arbitrary Units", ylab = "Depth (m)")
  
  axis(1)
  axis(2)
  
  plot(dec$m[,2], dec$dt, type = "l", ylim = c(max(cip1$dt), 0),
       xlim = c(100, -100), main = "Comp. 2", axes = F,
       xlab = "Arbitrary Units", ylab = "")
  
  axis(1)
  
  # plot(locpa3, dec$dt, ylim = c(max(cip1$dt), 0),
  #      type = "l", axes= F)
  #
  # axis(1)
  # axis(2)
  
  plot(filpre$xy, filpre$dt, type = "l", axes = F, main = "Taner",
       ylim = c(max(cip1$dt), 0), xlim = c(100, -100),
       xlab = "Arbitrary Units", ylab = "")
  
  axis(1)
  
  # plot(locpa4, dec$dt, ylim = c(max(cip1$dt), 0),
  #      type = "l", axes = F)
  #
  # axis(1)
  # axis(2)
}

pdfDisplay(g(), "MultiPrecession2", dir = getwd(), track = F)







# Solution for amplitude modulation of precession ----

d2 <- La04_pre_0_20

dt2 <- d2$age
xy2 <- d2$xy - mean(d2$xy)

norm <- normalise(m = xy2, dt = dt2)

amp <- norm$a - mean(norm$a)

# ----

fil1 <- taner(data.frame(dt = dt2, xy = amp),
              flow = 1/160, fhigh = 1/70, genplot = F)

fil2 <- taner(data.frame(dt = dt2, xy = amp),
              flow = 1/526, fhigh = 1/285, genplot = F)

fil3 <- taner(data.frame(dt = dt2, xy = amp),
              flow = 1/1400, fhigh = 1/800, genplot = F)

fil4 <- taner(data.frame(dt = dt2, xy = amp),
              flow = 1/2000, fhigh = 1/3000, genplot = F)

resi <- as.numeric(amp - fil1$X1 - fil2$X1 - fil3$X1 - fil4$X1)

mi <- as.matrix(cbind(fil1$X1, fil2$X1, fil3$X1, fil4$X1))

ampe <- as.emd(xy = amp, dt = dt2, 
               imf = mi, res = resi)

plot_emd(ampe, track = F, name = "Precession Amplitude 2",
         # adapt.axis = T,
         s = list(type = "l", lwd = 2),
         i = list(type = "l", lwd = 2) #, dtlim = c(9000, 5000)
)

check.emd(ampe, xy = amp)

g <- function()
{
  
  par(mfrow = c(1,9))
  
  xlim <- c(-0.03, 0.03)
  
  plot(norm$a, dt2, type = "l", ylim = ylim, xlim = xlim + mean(norm$a),
       axes = F, ylab = "Age (kyr)", xlab = "Climatic precession")
  abline(h  = c(6000, 8000))
  
  axis(2)
  axis(1)
  
  plot(fil1$X1, dt2, type = "l", ylim = ylim, xlim = xlim, axes = F,
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(-6000, -8000))
  
  axis(1)
  
  plot(fil2$X1, dt2, type = "l", ylim =ylim, xlim = xlim, axes = F,
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(6000, 8000))
  
  axis(1)
  
  plot(fil3$X1, dt2, type = "l", ylim = ylim, xlim = xlim, axes = F,
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(6000, 8000))
  
  axis(1)
  
  plot(fil3$X1 * 3, dt2, type = "l", ylim = ylim, xlim = xlim, axes = F,
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(6000, 8000))
  
  axis(1)
  
  plot(fil4$X1, dt2, type = "l", ylim = ylim, xlim = xlim, axes = F,
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(6000, 8000))
  
  axis(1)
  
  plot(fil4$X1 * 6, dt2, type = "l", ylim = ylim, xlim = xlim, axes = F,
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(6000, 8000))
  
  axis(1)
  
  plot(fil3$X1, dt2, type = "l", axes = F,
       ylim = c(20000,0),
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(5000, 6000, 8000, 9000))
  
  axis(1)
  axis(4)
  
  plot(fil4$X1, dt2, type = "l", axes = F,
       ylim = c(20000,0),
       ylab = "", xlab = "Climatic precession")
  abline(h  = c(5000, 6000, 8000, 9000))
  
  axis(1)
  axis(4)
  
}

pdfDisplay(g(), "Precession Amplitude", dir = getwd(), track = F, width = 25)











